## Chapter 14 

## Sec 14.1: Graphical User Interfaces to R 
## ss 14.1.1: The R Commander Graphical User Interface 
##                     Installing the R Commander 
install.packages("Rcmdr", dependencies=TRUE) 

## ss 14.1.2: Basics of the R Commander's interface 
library(Rcmdr) 

## ss 14.1.3: The \textit{rattle} GUI 
library(rattle} 
rattle() 

## Sec 14.2: Working Directories, Workspaces and the Search List 

## ss 14.2.1: *The search path 
search() 

## ss 14.2.2: Workspace management 
xy <- matrix(rnorm(60000), nrow=600) 
xy.rowrange <- apply(xy, 1, range) 
save(xy, xy.rowrange, file="xy.rda") 
rm(xy, xy.rowrange) 
## attach("c:/r/ch1/.RData")  # Microsoft Windows systems 
## ls(pos=2)                  # Check names of available objects 

##          Changing the Working Directory and/or Workspace 
## ss 14.2.3: Utility functions 
dir()                # List files in the working directory 
file.choose()        # Choose a file interactively 
sessionInfo()        # Print version numbers for R and for 
                     # attached packages 
R.home()             # Give the path to the R home directory 
.Library             # Path to the default library. 
Sys.getenv()         # Get settings of environment variables 
dir() 
Sys.getenv("R_HOME") 

## Sec 14.3: Data Input and Output 
library(DAAG) 
datafile("oneBadRow") 
datafile("scan-demo") 

## ss 14.3.1: Input of data 
##   Issues for data input with \texttt{read.table()} and variants 
##                   Tracking errors in input data 
nfields <- count.fields("oneBadRow.txt") 
nfields            # Number of fields, for each row 
(1:length(nfields))[nfields == 4] 

##  One character string for each input row -- \texttt{readLines()} 
readLines("oneBadRow.txt", n=3)    # First 3 lines 
readLines("oneBadRow.txt", n=-1)   # All lines 

##                     Input of fixed format data 
##                  Input of large rectangular files 
scan("oneBadRow.txt", skip = 1, quiet= TRUE) 
data.frame(scan("scan-demo.txt", skip = 1, quiet= TRUE, 

##            Example -- input of the Boston housing data 
readLines("bostonc.txt", n=11)[10:11] 
strsplit(readLines("bostonc.txt", n=11)[10:11], split="\t") 
boston <- scan("bostonc.txt", n=-1, sep="\t", skip=10, 
               what=c(list(1,""), as.list(rep(1,19)))) 
colnams <- scan("bostonc.txt", skip=9, n=21, what="") 
boston <- data.frame(boston) 
names(boston) <- colnams 
boston <- read.table("bostonc.txt", sep="\t", skip=9,  
                     comment.char="", header=TRUE) 

##                        Database connections 
## ss 14.3.2: Data output 
##                       Output of data frames 
write.table(fossilfuel, file="fuel.txt") 

##               Redirection of screen output to a file 
fossilfuel         # Below, this will be written to the file 
sink("fuel2.txt") 
fossilfuel         # NB: No output on screen 
sink() 

##             Output to a file using {\em\texttt{cat()}} 

## Sec 14.4: Functions and operators -- Some Further Details 
##            Issues for the writing and use of functions 
## ss 14.4.1: Function arguments 
##                 The {\em\texttt{args()}} function 
args(write.table)  # Version 2.2.1 of R 

##                     The \texttt{...} argument 
x <- c(1,5,7); z <- c(4,9,10,NA); u <- 1:10 
# Now do calculations that use x, z and u 
rm(x, z, u) 

## ss 14.4.2: Character string and vector functions 
substring("abracadabra",3, 8) # Extract characters 3 to 8 
nchar("abracadabra")          # Count the number of characters 
strsplit("abracadabra", "r")  # Split wherever "r" appears 
strsplit("abcd", split="")    # Split into separate characters 
paste("ab","c","d", sep="")   # Join together 
paste(c("a","b","c"), collapse="")  # Join vector elements 

## ss 14.4.3: Anonymous functions 
ssfun <- function(x)sum(x^2) 
sapply(elastic1, ssfun)   # elastic1 is from the DAAG package 
sapply(elastic1, function(x)sum(x^2)) 
growthfun <- function(x)(x[9] - x[1])/x[1] 
sapply(austpop[, -c(1,10)], growthfun) 
sapply(austpop[, -c(1,10)], function(x)(x[9] - x[1])/x[1]) 

## ss 14.4.4: Functions for working with dates (and times) 
# Electricity Billing Dates 
dd <- as.Date(c("2003-08-24","2003-11-23","2004-02-22","2004-05-03")) 
diff(dd) 
as.Date("1/1/1960", format="%d/%m/%Y") 
as.Date("1:12:1960",format="%d:%m:%Y") 
as.Date("1960-12-1")-as.Date("1960-1-1") 
as.Date("31/12/1960","%d/%m/%Y") 
as.integer(as.Date("1/1/1970","%d/%m/%Y")) 
as.integer(as.Date("1/1/2000","%d/%m/%Y")) 
dec1 <- as.Date("2004-12-1") 
format(dec1, format="%b %d %Y") 
format(dec1, format="%a %b %d %Y") 
## Labeling of graph: data frame jobs (DAAG) 
Jobs <- stack(jobs, select=-7) 
startofmonth <- seq(from=as.Date("1Jan1995", format="%d%b%Y"),  
                    by="1 month", length=24) 
Jobs$Date <- rep(startofmonth, 6) 
names(Jobs) <- c("Number", "Province", "Date") 
atdates <- seq(from=as.Date("1Jan1995", format="%d%b%Y"),  
               by="3 month", length=8) 
datelabs <- format(atdates, "%b%y") 
xyplot(Number ~ Date, groups=Province, data=Jobs, 
       scale=list(x=list(at=atdates, labels=datelabs)), 
       auto.key=list(columns=6, between=1)) 

## ss 14.4.5: Creating Groups 
library(MASS) 
catBP <- cut(Pima.tr2$bp, breaks=4) 
table(catBP, exclude=NULL) 

## ss 14.4.6: Logical operators 
c(TRUE, TRUE, FALSE) & c(FALSE, TRUE, FALSE) 
c(TRUE, TRUE, FALSE) && c(FALSE, TRUE, FALSE) 

##               A vector form of \texttt{if} statement 
## ss 14.4.7: Operators are functions 
"+"(2, 3) 

## Sec 14.5: Factors 
fac <- factor(c("c", "b", "a")) 
for (i in fac) print(i) 

##                          Ordered factors 
stress.level <- rep(c("low","medium","high"), 2) 
ordf.stress <- ordered(stress.level, levels=c("low", "medium", 
"high")) 
ordf.stress 
ordf.stress < "medium" 
ordf.stress >= "medium" 

##                          Factor contrasts 
contr.treatment(3) 
contr.sum(3) 
cities <- factor(c("Melbourne","Sydney","Adelaide")) 
contr.sum(cities) 

##      *Tests for main effects in the presence of interactions? 

## Sec 14.6: Missing Values 
x <- c(1, 6, 2, NA) 
is.na(x)           # TRUE for NAs, otherwise FALSE  
x == NA            # All elements are set to NA 
NA == NA 
x <- c(1, NA, 6, 2, 10) 
x > 4 # The second element will be NA 
x[x>4] # NB: This generates a vector of length 3 
x[x > 4] <- c(101, 102) 
x[!is.na(x) & x > 4] <- c(101, 102) 
x 

##  Counting and identifying {\rm NA}s -- the use of \texttt{table()} 
with(nswdemo, table(trt, re74>0, useNA="ifany")) 
fac_re74 <- with(nswdemo, factor(re74>0, exclude=NULL)) 
levels(fac_re74) 
levels(fac_re74) <- c("0", "gt0", "<NA>") 
with(nswdemo, table(trt, fac_re74)) 

##                         The removal of NAs 
##              {\em\texttt{NA}}s in modeling functions 
options()$na.action # Version 2.1.0, following startup 

##             Sorting and ordering, where there are NAs 
x <- c(1, 20,  2, NA, 22) 
order(x)           # By default, na.last=TRUE 
x[order(x)] 
sort(x)            # By default na.last=NA 
sort(x, na.last=TRUE) 

## Sec 14.7: {\kern-4pt}$^*$ Matrices and Arrays 
xx <- matrix(1:6, ncol=3)  # Equivalently, enter 
xx 
dim(xx) 
## Use as.vector() 
x <- as.vector(xx) 
## Alternatively, directly remove the dimension attribute 
x <- xx 
dim(x) <- NULL 

##                   The extraction of submatrices 
x34 <- matrix(1:12, ncol=4) 
x34 
x34[2:3, c(1,4)]     # Extract rows 2 & 3 and columns 1 & 4 
x34[-2, ]            # Extract all rows except the second 
x34[-2, -3]          # Omit row 2 and column 3 
x34[2, ]                # The dimension attribute is dropped 
x34[2, , drop=FALSE]    # Retain the dimension attribute 

##         Conversion of data frames and tables into matrices 
## ss 14.7.1: Matrix arithmetic 
## Set up example matrices G, H and B 
G <- matrix(1:12, nrow=4); H <- matrix(112:101, nrow=4); B <- 1:3 
G + H             # Element-wise addition (X & Y both n by m) 
G * H             # Element-wise multiplication 
G %*% B           # Matrix multiplication (X is n by k; B is k by m) 
## Set up example matrices X (square, full rank) and Y 
X <- matrix(c(1,1,1, -1,0,1, 2,4,2), nrow=3) 
Y <- matrix(1:6, nrow=3) 
solve(X, Y)       # Solve XB = Y (X must be square) 

##                      Computational Efficiency 
xy <- matrix(rnorm(500000),ncol=50) 
dim(xy) 
system.time(xy+1) 
xy.df <- data.frame(xy) 
system.time(xy.df+1) 

## ss 14.7.2: Outer products 
outer(1:4, 1:10) 
rbgshades <- outer(c("red","blue","green"), c("",paste(1:4)), 
                   function(x,y)paste(x,y, sep="")) 
rbgshades          # Display the matrix 
plot(rep(0:4, rep(3,5)), rep(1:3, 5), col=rbgshades, pch=15, cex=8) 

## ss 14.7.3: Arrays 
x <- 1:24 
dim(x) <- c(4, 6) 
x 
dim(x) <- c(3, 4, 2) 
x 

## Sec 14.8: Manipulations with Lists, Data Frames and Matrices 

## ss 14.8.1: Lists -- an extension of the notion of ``vector'' 
list1 <- list(1:3, 15, c("win","susan")) 
list1[c(3,1,2)] 
## Return list whose only element is the vector c("win","susan") 
list1[3] 
## Return the vector c("win","susan") 
list1[[3]] 

##                  The dual identity of data frames 
xyz <- data.frame(x=1:4, y=11:14, z=I(letters[1:4])) 
xyz[1, ]            # Returns a data frame, i.e., a list) 
xyz[1, , drop=TRUE] 
unlist(xyz[1, ])    # Returns, here, a vector of atomic mode 
unlist(xyz[1, , drop=TRUE]) 

## ss 14.8.2: Changing the shape of data frames (or matrices) 
## ss 14.8.3: The {\em \texttt{reshape()}} function 
library(reshape) 
Jobs <- melt(jobs, measure.vars=names(jobs)[1:6],  
head(Jobs, 3) 
jobsBack <- cast(Jobs, Date ~ Region) 
> head(jobsBack,3) 
      Date   BC Alberta Prairies Ontario Quebec Atlantic 
1 95.00000 1752    1366      982    5239   3196      947 
2 95.08333 1737    1369      981    5233   3205      946 
3 95.16667 1765    1380      984    5212   3191      954 

## ss 14.8.4: {\kern-4pt}$^*$ Merging data frames -- {\em\texttt{merge()}} 
new.Cars93 <- merge(x=Cars93, y=Cars93.summary[, "abbrev", drop=F], 
                    by.x="Type", by.y="row.names") 

## ss 14.8.5: Joining data frames, matrices and vectors -- {\em\texttt{cbind()}} 
##          Conversion of tables and arrays into data frames 
## ss 14.8.6: The {\em\texttt{apply}} family of functions 
##                The {\em\texttt{tapply()}} function 
## Compare tapply() with aggregate(): data frame kiwishade (DAAG) 
with(kiwishade, tapply(yield, INDEX=list(block, shade), FUN=mean)) 
with(kiwishade, aggregate(yield, by=list(block, shade), FUN=mean)) 

##                 The {\em\texttt{apply()}} function 
##  The functions {\em\texttt{lapply()}} and {\em\texttt{sapply()}} 
## Uses data frame rainforest (DAAG) 
sapply(rainforest[, -7], range, na.rm=TRUE) 

## ss 14.8.7: Splitting vectors and data frames into lists -- {\em\texttt{split()}} 
## Split values of HeadWt (data frame cabbages) by Date 
library(MASS) 
with(cabbages, split(HeadWt, Date) 
split(cabbages[, -1], cabbages$Date)    # Split remaining columns 
                                        # by levels of Date 

## ss 14.8.8: Multivariate time series 
jobts <- ts(jobs[,1:6], start=1995, frequency=12) 
colnames(jobts) 
## Subseries through to the third month of 1995 
window(jobts, end=1995+2/12) 
 # Rows are 1995+0/12, 1990+1/12, 1990+2/12 
plot(jobts, plot.type="single")     # Use one panel for all 
plot(jobts, plot.type="multiple")   # Separate panels. 
## Alternative code for Figure 2.9A; plot as time series 
plot(jobts, plot.type="single", xlim=c(1995,1997.2), lty=1:6, log="y", 
     xaxt="n", xlab="", ylab="Number of Jobs") 
## Move label positions so that labels do not overlap 
ylast <- bounce(window(jobts, 1996+11/12), d=strheight("O"), log=TRUE) 
 # bounce() is from DAAG 
text(rep(1996+11/12,6), ylast, colnames(ylast), pos=4) 
datlab <- format(seq(from=as.Date("1Jan1995", format="%d%b%Y"), by="3 month",  
                 length=8), "%b%Y") 
axis(1, at=seq(from=1995, by=0.25, length=8), datlab)  

## Sec 14.9: Classes and Methods 

##                       S3 methods and classes 
print 

## ss 14.9.1: Printing and summarizing model objects 
elastic.lm <- lm(distance ~ stretch, data=elasticband) 
elastic.sum <- summary(elastic.lm) 

## ss 14.9.2: Extracting information from model objects 
names(elastic.lm) 
elastic.lm$coefficients 
elastic.lm[["coefficients"]] 
elastic.lm[[1]] 
coef(elastic.lm) 

## ss 14.9.3: S4 classes and methods 
x <- 1:5 
class(x) <- "lm"   # Inappropriate assignment of class 
print(x) 
library(lme4) 
hp.lmList <- lmList(o2 ~ wattsPerKg | id, data=humanpower1) 
slotNames(hp.lmList) 
slot(hp.lmList, "call") 
hp.lmList@call 

## Sec 14.10: Manipulation of Language Constructs 

## ss 14.10.1: Model and graphics formulae 
plot.mtcars <- function(xvar="disp", yvar="hp"){ 
    attach(mtcars) 
    mt.txt <- paste(yvar, "~", xvar) 
    plot(formula(mt.txt)) 
    } 
> ## Plot using data frame mtcars (datasets) 
> names(mtcars) 
 [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   
[10] "gear" "carb" 
> plot.mtcars() 
> plot.mtcars(xvar="disp", yvar="mpg") 

##         Extraction of Variable Names from Formula Objects 
all.vars(mpg ~ disp) 
plot.mtcars <- function(form = mpg~disp){ 
   yvar <- all.vars(form)[1] 
   xvar <- all.vars(form)[2] 
   ## Include information that allows a meaningful label 
   mtcars.info <- c(mpg= "Miles/(US) gallon",  
                    cyl= "Number of cylinders",  
                    disp= "Displacement (cu.in.)",  
                    hp= "Gross horsepower",  
                    drat= "Rear axle ratio",  
                    wt= "Weight (lb/1000)",  
                    qsec= "1/4 mile time",  
                    vs= "V/S",  
                    am= "Transmission (0 = automatic, 1 = manual)",  
                    gear= "Number of forward gears",  
                    carb= "Number of carburettors") 
   xlab <- mtcars.info[xvar] 
   ylab <- mtcars.info[yvar] 
   plot(form, xlab=xlab, ylab=ylab) 
} 

## ss 14.10.2: The use of a list to pass arguments 
mean(rnorm(20)) 
do.call("mean", args=list(x=rnorm(20))) 
simulate.distribution <- 
  function(distn="weibull", parms=list(n=10)){  
    ## Simulates one of: weibull, gaussian, logistic 
    if(distn=="weibull" & !("shape" %in% names(parms))) 
      parms <- c(shape=1, parms) 
    ## weibull requires a default shape argument.  
    ## =====================================================  
    ## Choose the function that will generate the random sample  
    rfun <- switch(distn,   
                   weibull = rweibull,  
                   normal = rnorm,  
                   logistic= rlogis)  
    ## Call rfun(). Use of do.call() makes it possible to give 
    ## the argument list as a list of named values.  
    ## Pass shape argument (or NULL), plus n = # of numbers 
    do.call("rfun", args=parms)  
  } 
simulate.distribution() 
plot(density(simulate.distribution("normal", parms=list(n=100)))) 
plot(density(simulate.distribution("weibull",  
             parms=list(n=100, scale=0.5)))) 
mean.call <- call("mean", x=rnorm(5)) 
eval(mean.call) 
eval(mean.call) 
mean.call 

## ss 14.10.3: Expressions 
local(a+b*x+c*x^2, envir=list(x=1:4, a=3, b=5, c=1)) 

## ss 14.10.4: Environments 
test <- function(){  
fname <- as.character(sys.call())  
fname   
} 
test() 
newtest <- test    # Create a copy with a different name 
newtest() 

##       Automatic naming of a file that holds function output 
hcopy <- 
    function(width=2.25, height=2.25, color=F, pointsize=8){ 
        funtxt <- sys.call(1) 
        fnam <- paste(funtxt, ".pdf", sep="") 
        print(paste("Output will be to the file '",  
              fnam, "'", sep="")) 
        pdf(file=fnam, width=width, height=height, pointsize= 
            pointsize) 
    } 
fig1 <- function(){ 
    hcopy()             # Call with default arguments 
    curve(sin, -pi, 2*pi) 
    dev.off() 
} 

## ss 14.10.5: Function environments, and lazy evaluation 
##                          Lazy evaluation 
lazyfoo <- function(x=4, y = x^2)y 
lazyfoo() 
lazyfoo(x=3) 
lazyfoo(y=x^3)     # We specify a value for y in the function call 
x <- 9             # Now, in the parent environment, x=9 
lazyfoo(y=x^3) 

##  Example -- a function that identifies objects added during asession 
dsetnames <- objects() 
additions <- function(objnames = dsetnames){ 
    newnames <- objects(envir=.GlobalEnv) 
    existing <- newnames %in% objnames 
    newnames[!existing] 
    } 
additions(dsetnames) 

## Sec 14.11: Creation of R Packages 
package.skeleton(name = "viewtemps") 
package.skeleton(name = "viewtemps", path="~/packages") 

##                             Namespaces 

## Sec 14.12: Document Preparation --- \texttt{Sweave()} and \texttt{xtable()} 
Sweave("sec1-1.Rnw") # actually, Sweave("sec1-1") is sufficient 

## Sec 14.13: Graphs in R 

## ss 14.13.1: Hardcopy graphics devices 
pdf(file="fossilfuel.pdf", width=6.5, height=6.5, pointsize=18) 
plot(Carbon ~ Year, data=fossilfuel, pch=16) 
dev.off() 
plot(carbon ~ year, data=fossilfuel, pch=16)  # Display graph 
## Now open pdf device and copy graph to it 
dev.copy(pdf, file="fossilcopy.pdf") 
dev.off()            # Close pdf device 

## ss 14.13.2: Multiple graphs on a single base graphics page 
## ss 14.13.3: Plotting characters, symbols, line types and colors 
##                           Font families 
##                               Colors 
##               Rectangles, polygons and other shapes 
##                  Contour and filled contour plots 
## ss 14.13.4: Formatting and plotting of text and equations 
##         Symbolic substitution of symbols in an expression 
library(DAAG) 
## Replace the symbol tx by its value 
specnam <- "Acmena smithii" 
plot(wood ~ dbh, data=rainforest, subset=species==specnam) 
title(main=substitute(italic(tx) * ": " * "wood vs dbh", 
                      list(tx=specnam))) 

##                  Plotting expressions in parallel 
## Sample from bivariate normal (x, y) with correlation rho 
rho <- 0.75; x <- rnorm(40); y <- rnorm(40) + rho/sqrt(1-rho^2)*x 
## Now map all values of x onto 3 integer values 
x0 <- cut(x, breaks=c(-Inf,-1,1,Inf)) 
plot(unclass(x0), y, xaxt="n") 
axis(1, at=1:3, labels=expression("(-Inf,-1]", "(-1,1]",  
                                  "(1, Inf]")) 
axis(1, at=1:3, labels=expression("(" * -infinity * ",  
         " * -1 * "]", "(" * -1 * ", 1]", "(1, " * infinity * "]")) 

##                 Symbolic substitution in parallel 
tiklab <- lapply(strsplit(levels(x0),"Inf"),  
              function(x)if(length(x)>1)substitute(x1*infinity*x2,  
                                   list(x1=x[1],x2=x[2]))} else x) 
axis(1, at=1:3, labels=as.expression(tiklab)) 

##        Symbolic substitution in parallel -- another example 
## Plot brain vs body: data frame Animals (MASS package) 
library(MASS) 
log10ticks <- with(Animals, pretty(log10(c(1000*body, brain)), 3)) 
tickpos <- 10^log10ticks 
plot(brain ~ I(1000*body), log="xy", data=Animals, xaxt="n",  
     yaxt="n", xlim=range(tickpos), ylim=range(tickpos), pty="s", 
     xlab="Body weight (g)", ylab=""Brain weight (g)") 
ticklabs <- lapply(round(log10ticks,2),  
                   function(x)substitute(10^a, list(a=x))) 
axis(1, at=tickpos, labels=as.expression(ticklabs)) 
axis(2, at=tickpos, labels=as.expression(ticklabs))   
## Line from robust regression; here must use log to base 10 
abline(lqs(log10(brain) ~ log10(1000*body), data=Animals))  
## Get details of fitted line, on a natural logarithmic scale 
ab <- lqs(log(brain) ~ log(1000*body), data=Animals)$coef 
a <- round(exp(ab[1]),3) 
b <- round(ab[2],3) 
legend("topleft", legend=substitute(hat(brain) == A*" "*body^B, 
                                    list(A=a, B=b)), bty="n") 

##              A further use for \texttt{substitute()} 
testfun <- function(x, y){ 
 xpar <- deparse(substitute(x)) 
 xpar} 
testfun(x = AnyValidName) 

## Sec 14.14: Lattice Graphics, and the \textit{grid} Package 

## ss 14.14.1: Lattice Graphics vs Base Graphics 
## ss 14.14.2: Groups within data, and/or columns in parallel 
## Simple version of plot 
grogplot <- xyplot(Beer+Spirit+Wine ~ Year | Country, data=grog, 
                   outer=FALSE, auto.key=list(columns=3)) 
## Enhance, and print enhanced code 
update(grogplot, ylim=c(0,5.5),  
       xlab="", ylab="Amount consumed (per person)", 
       par.settings=simpleTheme(pch=c(1,3,4))) 
## Footnote code 
## Update trellis object, then print 
frillyplot <- 
  update(grogplot, ylim=c(0,5.5), xlab="", ylab="Amount consumed (per person)", 
         par.settings=simpleTheme(pch=c(1,3,4))) 
print(frillyplot) 
xyplot(Beer+Spirit+Wine ~ Year, groups=Country, outer=TRUE,  
       data=grog, auto.key=list(columns=2) ) 

## ss 14.14.3: Lattice Parameters and Graphics Features 
##  Point, line and fill color settings, using \texttt{simpleTheme()} 
settings1 <- simpleTheme(pch = c(1,3,4), cex=1.25) 
settings2 <- simpleTheme(pch = c(1,3,4), alpha=0.5) 
grogplot0 <- xyplot(Beer+Spirit+Wine ~ Year | Country, outer=FALSE, 
                    data=grog, ylim=c(0,5.5)) 
grogplot <- update(grogplot0, par.settings=settings1) 
print(grogplot) 
trellis.par.set(settings2) 

##                   Beyond \texttt{simpleTheme()} 
names(trellis.par.get()) 
trellis.device(color=FALSE) 
show.settings() 
trellis.device(color=TRUE) 
show.settings() 
trellis.par.set(list(fontsize = list(text = 7, points = 4))) 

##      Parameters that affect axes, tick marks, and axis labels 
## Create a simplified version of the graphics object 
jobs.xyplot <- 
  xyplot(Ontario+Quebec+BC+Alberta+Prairies+Atlantic ~ Date,  
         data=jobs, type="b", layout=c(3,2), ylab="Number of jobs", 
         scales=list(y=list(relation="sliced", log=TRUE)),  
         outer=TRUE) 
## Update jobs.xyplot, with various enhancements 
ylabpos <- exp(pretty(log(unlist(jobs[,-7])), 100)) 
ylabels <- paste(round(ylabpos),"\n(", log(ylabpos), ")", sep="")  
## Create a date object 'startofmonth'; use this instead of 'Date' 
atdates <- seq(from=95, by=0.5, length=5) 
datelabs <- format(seq(from=as.Date("1Jan1995", format="%d%b%Y"),   
                   by="6 month", length=5), "%b%y") 
update(jobs.xyplot, xlab="", between=list(x=0.5, y=0.5), 
       scales=list(x=list(at=atdates, labels=datelabs), 
                   y=list(at=ylabpos, labels=ylabels), tck=0.6) ) 

## ss 14.14.4: A further example 
with(ais, table(sex,sport)) 
basic1 <-  xyplot(hc ~ rcc | sex, groups=sport[drop=TRUE],  
                  data=subset(ais, sport %in% c("B_Ball", "Swim")), 
                  ylab="Blood cell to plasma ratio (%)") 
parSet <- simpleTheme(pch = c(1,3), lty=1:2, lwd=1.5) 
xaxlab <- expression("Red cell count (10"^{12}*"."*L^{-1}*")") 
basic2 <- update(basic1, par.settings=parSet,  
                 auto.key=list(columns=2, lines=TRUE), 
                 scales=list(tck=0.5), xlab=xaxlab) 
print(update(basic2, type=c("p", "r"))) 

## ss 14.14.5: Keys -- \texttt{auto.key}, \texttt{key} \& \texttt{legend} 
## ss 14.14.6: Panel Functions and Interaction with Plots 
xyplot(ACT ~ year, data=austpop) 
xyplot(ACT ~ year, data=austpop, panel=panel.xyplot) 
update(basic2, 
    strip=strip.custom(factor.levels=c("Female","Male")), 
    # In place of level names c("f", "m"), use c("Female", "Male") 
    panel=function(x, y, groups, subscripts, ...){ 
         panel.superpose(x,y, groups=groups,  
                         subscripts=subscripts, ...) 
         b <- coef(lm(y ~ groups[subscripts] + x)) 
         lcol <- trellis.par.get()$superpose.line$col 
         lty <- trellis.par.get()$superpose.line$lty 
         panel.abline(b[1], b[3], col=lcol[1], lty=lty[1]) 
         panel.abline(b[1]+b[2], b[3], col=lcol[2], lty=lty[2]) 
         }) 
  panel=function(x, y, groups, subscripts, ...){ 
       panel.superpose(x,y, groups=groups,  
                    subscripts=subscripts, ...) 
       . . . . 
       panel.abline(b[1], b[3], col=lcol[1], lty=lty[1]) 
       panel.abline(b[1]+b[2], b[3], col=lcol[2],  
                    lty=lty[2]) 
       }) 
    b <- coef(lm(y ~ groups[subscripts] + x)) 
      # NB: groups (but not x and y) must be subscripted 

##   Interaction with lattice plots -- the \textit{playwith}package 
install.packages("playwith", dependencies=TRUE) 
library(DAAGxtras) 
library(playwith) 
playwith(xyplot(age ~ distance, data=hotspots), 
                labels=hotspots$name) 
gph <- xyplot(age ~ distance, data=hotspots) 
playwith(update(gph), labels=hotspots$name) 

## ss 14.14.7: Interaction with lattice plots -- focus, interact,                      unfocus 
##   Use of \texttt{panel.identify()} to label points interactively 
## Use of xyplot(): data frame tinting (DAAG) 
library(lattice) 
xyplot(it ~ csoa | sex, data=tinting) 
trellis.focus("panel", column=1, row=1) 
panel.identify(labels=as.character(tinting$target))     
## Now click near points as required. 
## Terminate by right clicking inside the panel. 
## Now interact with panel 2 
trellis.focus("panel", row=1, column=2) 
panel.identify(labels=as.character(tinting$target)) 
## Click, ..., and right click 

##            Use of \texttt{panel.text()} to label points 
xyplot(Brainwt ~ Bodywt, data=primates) 
trellis.focus("panel", row=1, column=1, clip.off=TRUE) 
xyetc <- trellis.panelArgs() 
panel.text( x=xyetc$x, y=xyetc$y, labels=row.names(primates), pos=3) 
trellis.unfocus() 

##          Non-interactive use of \texttt{trellis.focus()} 
## ss 14.14.8: Multiple lattice graphs on a graphics page 
## Two lattice graphs on one page: data frame cuckoos (DAAG) 
trellis.par.set(layout.heights=list(key.top=0.25, axis.top=0.5, 
                                    bottom.padding=0.15)) 
cuckoos.strip <- stripplot(species ~ length, xlab="", data=cuckoos) 
print(cuckoos.strip, position=c(0,.475,1,1)) 
  # Left bottom corner is (0,.475); right upper corner is (1,1) 
trellis.focus(name="toplevel") 
panel.text("A", x=0.05, y=0.95)   
  # Here, x=0.05 translates to x=unit(0.05,"npc"); the range is (0,1) 
trellis.unfocus() 
cuckoos.bw <- bwplot(species~length, xlab="Egg length (mm)", 
                     data=cuckoos)  
print(cuckoos.bw, newpage=FALSE, position=c(0,0,1,.525)) 
trellis.focus(name="toplevel") 
panel.text("B", x=0.05, y=0.95)   
trellis.unfocus() 

##     Use of \texttt{textGrob()} in the \texttt{legend} argument 
library(grid) 
stripplot(species ~ length, xlab="", data=cuckoos, 
          legend=list(top=list(fun=textGrob, 
                      args=list(label="A", x=0)))) 
  # Here, x=0 is equivalent to x=unit(0,"npc"); the range is (0,1) 

##                Overlaid plots with different scales 
library(latticeExtra) 
library(DAAGxtras) 
CO2smu <- with(edcCO2, supsmu(age, co2, span=.005)) 
tempsmu <- with(edcT, supsmu(Age, dT, span=.001)) 
co2_layer <- xyplot(y ~ I(-x), data=CO2smu, type="l", 
                    xlab="Years Before Present",  
                    ylab = expression(CO[2]*" (ppm)")) 
temp_layer <- xyplot(I(y+54.5) ~ I(-x), data=tempsmu, type="l", 
                     ylab=expression("Temperature ("*degree*C*")")) 
doubleYScale(co2_layer, temp_layer, add.ylab2 = TRUE) 

##          Base and trellis plots on the same graphics page 
plot(0:1, 0:1, type="n", bty="n", axes=FALSE, xlab="", ylab="") 
mtext(side=3, line=3, "Lattice bwplot (i.e., boxplot)") 
cuckoos.bw <- bwplot(species~length, data=cuckoos)  
print(cuckoos.bw, newpage=FALSE) 

## Sec 14.15: An Implementation of Wilkinson's \textit{Grammar of Graphics} 

## ss 14.15.1: Australian rain data 
library(DAAG) 
library(ggplot2) 
## Default loess smooth, with SE bands added. 
quickplot(Year, seRain, data=bomsoi, geom=c("point","smooth"), 
          span=0.1, xlab="", ylab="Av. rainfall, M-D basin") 

## ss 14.15.2: Physical measurements of Australian athletes 
## Overlay scatterplots with boxplots and with density contours 
quickplot(wt, ht, xlab="Weight (kg)", ylab="Height (cm)", data=ais, 
          geom=c("boxplot", "point", "density2d"),  
          facets = . ~ sex) 
## Two panels (columns): sexes have different colors and symbols 
quickplot(wt, ht, data=aisBS, type="point", 
          colour=sex, shape=sex, 
          facets = . ~ sport) 
## Single panel: distinguish sexes by colors; sports by symbols 
quickplot(wt, ht, data=aisBS, type="point", 
          colour=sex, shape=sport) 
aisBS <- subset(ais, sport %in% c("Row","Swim")) 
aisBS$sport <- factor(aisBS$sport) 
quickplot(wt, ht, data=aisBS, type="point", 
          colour=sex, shape=sex, size=I(2.5), 
          facets = . ~ sport) 
## Single panel: distinguish sexes by colors; sports by symbols 
quickplot(wt, ht, data=aisBS, type="point", 
          color=sex, shape=sport, size=I(2.5)) 
## Single panel: distinguish sexes by colors; sports by symbol size 
quickplot(wt, ht, data=aisBS, type="point", 
          color=sex, size=sport) 
## Change the base pointsize for text to 8 
theme_set(theme_gray(base_size=8))  # Gray theme 
theme_set(theme_bw(base_size=8))    # Black and white theme 
## Modify the pointsize 
update_geom_defaults("point", aes(cex=1.25)) 

## Sec 14.16: Dynamic Graphics -- the \textit{rgl} and \texttt{rggobi} packages 
library(DAAG) 
library(rgl) 
open3d(params=list(windowRect=c(0,0,360,360), zoom=0.8, cex=0.65)) 
bg3d("white") 
with(possum,  
     plot3d(earconch, taill, footlgth, col=s.numeric(site), 
            type="s", size=0.3)) 

## Sec 14.17: Further Reading 
citation() 
citation("DAAG") 

##                             Vignettes 
vignette()           # All vignettes in all installed packages 
vignette(package="grid") 
vignette("grid")    # Equivalent to vignette(topic="grid") 
## Use of split() and sapply(): data frame science (DAAG) 
with(science, sapply(split(school, PrivPub), 
                     function(x)length(unique(x)))) 
### Thanks to Markus Hegland (ANU), who wrote the initial version 
##1 Generate the data 
  cat("generate data \n") 
  n <- 800           # length of noise vector 
  m <- 100           # length of signal vector 
  xsignal <- runif(m) 
  sig <- 0.01 
  enoise <- rnorm(m)*sig 
  ysignal <- xsignal**2+enoise 
  maxys <- max(ysignal) 
  minys <- min(ysignal) 
  x <- c(runif(n), xsignal) 
  y <- c(runif(n)*(maxys-minys)+minys, ysignal) 
  # random permutation of the data vectors 
  iperm <- sample(seq(x)) 
  x <- x[iperm] 
  y <- y[iperm] 
  # normalize the data, i.e., scale x & y values to 
  # lie between 0 & 1 
  xn <- (x - min(x))/(max(x) - min(x)) 
  yn <- (y - min(y))/(max(y) - min(y)) 
##1 End 
 
##2 determine number of neighbors within 
# a distance  <= h = 1/sqrt(length(xn)) 
  nx <- length(xn) 
  # determine distance matrix 
  d <- sqrt( (matrix(xn, nx, nx) - t(matrix(xn, nx, nx)) )**2 + 
             (matrix(yn, nx, nx) - t(matrix(yn, nx, nx)) )**2 ) 
  h <- 1/sqrt(nx) 
  nnear <- apply(d <= h, 1, sum) 
##2 End 
 
##3 Plot data, with reconstructed signal overlaid. 
  cat("produce plots \n") 
  # identify the points which have many such neighbors 
  ns <- 8 
  plot(x,y) 
  points(x[nnear > ns], y[nnear > ns], col="red", pch=16) 
##3 End 
n <- 10000; system.time(sd(rnorm(n))) 

## Sec 14.18: References 

##                   References for further reading 

## Sec 14.19: References 
##                   References for further reading 
