## Chapter 13 

## Sec 13.1: Principal Component Scores in Regression 
## Principal components: data frame socsupport (DAAG) 
ss.pr1 <- princomp(as.matrix(na.omit(socsupport[, 9:19])), cor=TRUE) 
pairs(ss.pr1$scores[, 1:3]) 
sort(-ss.pr1$scores[,1], decr=TRUE)[1:10]   # Note the outlier 
## Alternative to pairs(), using the lattice function splom() 
splom(~ss.pr1$scores[, 1:3]) 
not.na <- complete.cases(socsupport[, 9:19]) 
not.na[36] <- FALSE 
ss.pr <- princomp(as.matrix(socsupport[not.na, 9:19]), cor=TRUE) 
summary(ss.pr)     # Examine the contributions of the components 
ss.lm <- lm(BDI[not.na] ~ ss.pr$scores[, 1:6], data=socsupport) 
summary(ss.lm)$coef 
ss.pr$loadings[, 1] 
## Footnote code 
attach(socsupport) 
plot(BDI[not.na] ~ ss.pr$scores[ ,1], col=as.numeric(gender[not.na]),  
     pch=as.numeric(gender[not.na]), xlab ="1st principal component",  
     ylab="BDI") 
topleft <- par()$usr[c(1,4)] 
legend(topleft[1], topleft[2], col=1:2, pch=1:2, legend=levels(gender)) 
detach(socsupport) 

## Sec 13.2: {\kern-4pt}$^*$ Propensity Scores in Regression Comparisons -- Labor Training Data 

##                      The labor training data 

##                  Summary information on the data 
## Footnote code 
showprop <- 
  function(dframe=psid1, facCols=4:7, zeroCols=9:10){ 
    info <- numeric(length(facCols)+length(zeroCols)) 
    info[1:length(facCols)] <- sapply(dframe[,facCols], function(x){ 
      z <- table(x); z[2]/sum(z)}) 
    info[-(1:length(facCols))] <- sapply(dframe[,zeroCols], function(x) 
      sum(x>0)/sum(!is.na(x))) 
    info 
  } 
## Create matrix to hold resuilt 
propmat <- matrix(0, ncol=6, nrow=8) 
dimnames(propmat) <- 
  list(c("psid1", "psid2", "psid3", "cps1", "cps2", "cps3",  
          "nsw-ctl", "nsw-trt"), names(nswdemo)[c(4:7, 9:10)]) 
## Run function 
for(k in 1:8){ 
  dframe <- switch(k, psid1, psid2, psid3, cps1, cps2, cps3,  
                   subset(nswdemo, trt==0), subset(nswdemo, trt==1)) 
  propmat[k,] <- showprop(dframe) 
} 
vnames <- c("trt","educ","age","re75","re78") 
nsw <- rbind(psid1, subset(nswdemo, trt==1)) 
## Check minimum non-zero values of re75 and re78 
round(sapply(nsw[,c("re75","re78")], function(x)unique(sort(x))[2])) 
nsw[,c("re75","re78")] <- log(nsw[,c("re75","re78")] + 30) 
lab <- c(vnames[2:3], paste("log\n", vnames[-(1:3)], "+", 30)) 
nsw$trt <- factor(nsw$trt, labels=c("Control (psid1)","Treatment")) 
splom(~ nsw[,vnames[-1]], type=c("p","smooth"), groups=nsw$trt,  
      varnames=lab, auto.key=list(columns=2)) 

## ss 13.2.1: Regression comparisons 
##              Issues for the use of regression methods 
##                      Regression calculations 
nsw$fac74 <- with(nswdemo, factor(re74>0, exclude=NULL)) 
table(nsw$fac74)     # Check the order of the levels 
levels(nsw$fac74) <- c("0","gt0","<NA>") 
nswlm <- 
  function(control=psid1, df1=2, log78=TRUE, offset=30, printit=TRUE){ 
    nsw0 <- rbind(control, subset(nswdemo, trt==1)) 
    nsw0$fac74 <- factor(nsw0$re74>0, exclude=NULL) 
    levels(nsw0$fac74) <- c("0","gt0","<NA>") 
    if(log78) nsw.lm <- lm(log(re78+offset) ~ trt + ns(age,df1) + 
                           ns(educ,df1) + black + hisp + fac74 + 
                           ns(log(re75+offset),df1), data=nsw0) else 
    nsw.lm <- lm(re78 ~ trt + ns(age,df1) + ns(educ,df1) + black +  
                           hisp + fac74 + ns(log(re75+offset),df1),  
                           data=nsw0)  
    if(printit) print(summary(nsw.lm)) 
    trtvec <- unlist(summary(nsw.lm)$coef["trt", 1:2]) 
    trtEst <- c(trtvec[1], c(trtvec[1]+trtvec[2]*c(-1.96,1.96))) 
    if(log78) { 
      trtEst <- c(trtEst[1], exp(trtEst[1]), exp(trtEst[-1])) 
      names(trtEst)=c("Est.","exp(Est.)","CIlower","CIupper") 
    } else 
    names(trtEst)=c("Est.","CIlower","CIupper") 
    if(printit) print(trtEst) 
    invisible(list(obj=nsw.lm, est=trtEst)) 
  } 
## Try for example 
nsw.lm1 <- nswlm(control=psdi1)$nsw.lm   
nswlm(control=subset(nswdemo, trt=0)) 
nswlm(control=psdi1, log78=FALSE)  
for (z in list(psid1,psid2,psid3,cps1,cps2,cps3)) 
  print(nswlm(control=z, printit=FALSE)$est) 

## ss 13.2.2: A strategy that uses propensity scores 
##               Derivation and investigation of scores 
## Use the dataset nsw that combines psid1 with exptl trt obsns 
nsw.rf <- randomForest(trt ~ ., data=nsw[, -c(7:8,10)], 
                       sampsize=c(297,297)) 
## NB: Use of equal bootstrap sample sizes (= 297 = number of  
## treatment observations) gives the two groups equal prior weight. 
nsw.rf 
library(MASS) 
library(splines) 
nsw.lda <- lda(trt ~ ns(age,2) + ns(educ,2) + black + hisp +  
               fac74 +  ns(log(re75+30),3),  
               CV=TRUE, prior=c(.5,.5), data=nsw)  
tab <- table(nsw.lda$class, nsw$trt) 
1 - sum(tab[row(tab)==col(tab)])/sum(tab) 
logit <- function(p, offset=0.001)log((p+offset)/(1+offset-p)) 
trt <-  nsw$trt 
## NB: Derive scores by a logit transform of probabilities 
lprob.rf <- logit(predict(nsw.rf, type="prob")[,2]) 
overlapDensity(lprob.rf[trt==0], lprob.rf[trt==1], ratio=c(1/20, 20)) 
nsw.lda <- lda(trt ~ ns(age,2) + ns(educ,2) + black + hisp + fac74 + 
               ns(log(re75+30),3), prior=c(.5,.5), data=nsw)  
lprob.lda <- logit(nsw.lda$posterior[,2]) 
overlapDensity(lprob.lda[trt==0], lprob.lda[trt==1], ratio=c(1/20, 20)) 
nswa <- nsw[lprob.rf>-1.5, ] 
nswa.rf <- randomForest(trt ~ ., data=nswa[, -c(7:8,10)]) 
proba.rf <- predict(nswa.rf, type="prob")[,2] 
lproba.rf <- logit(proba.rf) 

##                  Checks on the propensity scores 
xyplot(age + educ + log(re75+30) ~ lproba.rf, groups=trt, layout=c(3,1),  
       data=nswa, type=c("p","smooth"), span=0.4, aspect=1, 
       par.settings=simpleTheme(lwd=c(2,1.5), col=c("gray", "black"), 
         pch=c(20,3), cex=0.5, lty=c("solid","21")), 
       scales=list(y=list(relation="free"), tck=0.5), 
       auto.key=list(columns=2, points=TRUE, lines=TRUE, 
         text=c("psid1 controls", "experimental treatment")), 
       xlab="Scores, derived using randomForest()")   

##    Use of proximities to give a two-dimensional representation 
## Repeat randomForest calculation, now with proximities 
nswa.rf <- randomForest(trt ~ ., data=nswa[, -c(7:8,10)], 
                        proximity=TRUE) 
## Subtract proximities from 1.0, and use as "distances" 
dmat <- 1-nswa.rf$proximity 
dmat[dmat==0] <- 0.001 
## From "distances", derive an ordination. 
pts <- cmdscale(dmat) 
rf.mds <- isoMDS(dmat, pts) 
ordScores <- rf.mds$points 
cutpts <- c(0, round(quantile(proba.rf, c(1/3,2/3)), 2), 1). 
cutp <- cut(lproba.rf, breaks=cutpts, include.lowest=TRUE) 
xyplot(ordScores[,2] ~ ordScores[,1]|cutp, groups=nswa$trt,  
               xlab="Co-ordinate 1", ylab="Co-ordinate 2",  
               auto.key=list(columns=2), aspect=1, 
               par.settings=simpleTheme(col=c("black","gray"),  
               pch=c(1,3))) 

##   Probability of non-zero earnings -- analysis using the scores 
lproba.rf <- logit(proba.rf) 
 rf.glm <- glm(I(re78>0) ~ ns(lproba.rf,2)+trt, data=nswa,  
               family=binomial) 
 summary(rf.glm) 

##   Distribution of non-zero earnings -- analysis using the scores 
rf.lm <- lm(log(re78+30) ~ ns(lproba.rf,2)+trt, data=nswa,  
             subset = re78>0) 
summary(rf.lm) 
round(summary(lm(log(re78+30) ~ trt, data=nswdemo,  
      subset=re78>0))$coef, 4) %$ 

## Sec 13.3: Further Reading 
